@@ -3,8 +3,8 @@
 ;;
 ;; MODULE      : init-gnuplot.scm
 ;; DESCRIPTION : Initialize GNUplot plugin
-;; COPYRIGHT   : (C) 1999  Joris van der Hoeven
-;;                   2019  Darcy Shen
+;; COPYRIGHT   : (C) 1999       Joris van der Hoeven
+;;                   2019-2020  Darcy Shen
 ;;
 ;; This software falls under the GNU general public license version 3 or later.
 ;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
@@ -12,6 +12,13 @@
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define (python-command)
+  (if (url-exists-in-path? "python3") "python3" "python2"))
+
+(define (python-exists?)
+  (or (url-exists-in-path? "python3")
+      (url-exists-in-path? "python2")))
+
 (define (gnuplot-serialize lan t)
     (with u (pre-serialize lan t)
       (with s (texmacs->code (stree->tree u) "SourceCode")
@@ -19,17 +26,17 @@
 
 (define (gnuplot-launcher)
   (if (url-exists? "$TEXMACS_HOME_PATH/plugins/tmpy")
-      `((:launch ,(string-append "python \""
+      `((:launch ,(string-append (python-command) " \""
                                  (getenv "TEXMACS_HOME_PATH")
                                  "/plugins/tmpy/session/tm_gnuplot.py\"")))
-      `((:launch ,(string-append "python \""
+      `((:launch ,(string-append (python-command) " \""
                                  (getenv "TEXMACS_PATH")
                                  "/plugins/tmpy/session/tm_gnuplot.py\"")))))
 
 
 (plugin-configure gnuplot
   (:require (url-exists-in-path? "gnuplot"))
-  (:require (url-exists-in-path? "python"))
+  (:require (python-exists?))
   ,@(gnuplot-launcher)
   (:serializer ,gnuplot-serialize)
   (:session "Gnuplot")
